@@ -7,6 +7,8 @@ from Crypto.Hash import HMAC, SHA1
 from Crypto.Util import Counter
 from Crypto.Protocol.KDF import PBKDF2
 
+from stream_inflate import stream_inflate64
+
 
 def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
     local_file_header_signature = b'\x50\x4b\x03\x04'
@@ -68,7 +70,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
         return _yield_all, _get_num, _return_unused
 
-    def get_dummy_decompressor(num_bytes):
+    def get_decompressor_none(num_bytes):
         num_decompressed = 0
         num_unused = 0
 
@@ -87,7 +89,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
         return _decompress, _is_done, _num_unused
 
-    def get_deflate_decompressor():
+    def get_decompressor_deflate():
         dobj = zlib.decompressobj(wbits=-zlib.MAX_WBITS)
 
         def _decompress_single(compressed_chunk):
@@ -114,6 +116,14 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
         return _decompress, _is_done, _num_unused
 
+    def get_decompressor_deflate64():
+        uncompressed_chunks, is_done, num_bytes_unconsumed = stream_inflate64()
+
+        def _decompress(compressed_chunk):
+            yield from uncompressed_chunks((compressed_chunk,))
+
+        return _decompress, is_done, num_bytes_unconsumed
+
     def yield_file(yield_all, get_num, return_unused):
 
         def get_flag_bits(flags):
@@ -146,7 +156,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
             return value
 
-        def weak_decrypt_decompress(chunks, decompress, is_done, num_unused):
+        def decrypt_weak_decompress(chunks, decompress, is_done, num_unused):
             key_0 = 305419896
             key_1 = 591751049
             key_2 = 878082192
@@ -180,7 +190,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
 
             return_unused(num_unused())
 
-        def aes_decrypt_decompress(chunks, decompress, is_done, num_unused, key_length_raw):
+        def decrypt_aes_decompress(chunks, decompress, is_done, num_unused, key_length_raw):
             try:
                 key_length, salt_length = {1: (16, 8), 2: (24, 12), 3: (32, 16)}[key_length_raw]
             except KeyError:
@@ -209,7 +219,7 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
             if get_num(10) != hmac.digest()[:10]:
                 raise HMACIntegrityError()
 
-        def no_decrypt_decompress(chunks, decompress, is_done, num_unused):
+        def decrypt_none_decompress(chunks, decompress, is_done, num_unused):
             while not is_done():
                 yield from decompress(next_or_truncated_error(chunks))
 
@@ -274,26 +284,27 @@ def stream_unzip(zipfile_chunks, password=None, chunk_size=65536):
             Struct('<H').unpack(aes_extra[5:7])[0] if is_aes_encrypted else \
             compression_raw
 
-        if compression not in (0, 8):
+        if compression not in (0, 8, 9):
             raise UnsupportedCompressionTypeError(compression)
 
-        is_zip64 = compressed_size_raw == zip64_compressed_size and uncompressed_size_raw == zip64_compressed_size
-        zip64_extra = get_extra_value(extra, is_zip64, zip64_size_signature, MissingZip64ExtraError, 16, TruncatedZip64ExtraError)
-
         has_data_descriptor = flag_bits[3]
+        is_zip64 = compressed_size_raw == zip64_compressed_size and uncompressed_size_raw == zip64_compressed_size
+        zip64_extra = get_extra_value(extra, not has_data_descriptor and is_zip64, zip64_size_signature, MissingZip64ExtraError, 16, TruncatedZip64ExtraError)
+
         uncompressed_size = \
-            None if has_data_descriptor and compression == 8 else \
+            None if has_data_descriptor and compression in (8, 9) else \
             Struct('<Q').unpack(zip64_extra[:8])[0] if is_zip64 else \
             uncompressed_size_raw
 
         decompressor = \
-            get_dummy_decompressor(uncompressed_size) if compression == 0 else \
-            get_deflate_decompressor()
+            get_decompressor_none(uncompressed_size) if compression == 0 else \
+            get_decompressor_deflate() if compression == 8 else \
+            get_decompressor_deflate64()
 
         decompressed_bytes = \
-            weak_decrypt_decompress(yield_all(), *decompressor) if is_weak_encrypted else \
-            aes_decrypt_decompress(yield_all(), *decompressor, key_length_raw=aes_extra[4]) if is_aes_encrypted else \
-            no_decrypt_decompress(yield_all(), *decompressor)
+            decrypt_weak_decompress(yield_all(), *decompressor) if is_weak_encrypted else \
+            decrypt_aes_decompress(yield_all(), *decompressor, key_length_raw=aes_extra[4]) if is_aes_encrypted else \
+            decrypt_none_decompress(yield_all(), *decompressor)
 
         get_crc_32_expected = \
             partial(get_crc_32_expected_from_data_descriptor, is_zip64) if has_data_descriptor else \
